unit Playlist;

{$IMPORTEDDATA OFF}

interface

type
  TPlaylist = class(TObject)
  private
    procedure AddOne(const Path: WideString);
    procedure AddDir(const Path: WideString);
  public
    Cur: Integer;
    Count: Integer;
    Items: array of WideString;
    constructor Create;
    procedure Add(const Path: WideString);
    procedure Clear;
    procedure Sort;
  end;

implementation

uses NLib, NSys, NCmpLogical;

constructor TPlaylist.Create;
begin
  Clear;
end;

procedure TPlaylist.Add(const Path: WideString);
begin
  if IsDir(Path) then
    AddDir(Path)
  else
    AddOne(Path);
end;

procedure TPlaylist.AddOne(const Path: WideString);
begin
  if Count = Length(Items) then 
    SetLength(Items, Count + Count div 4);
  Items[Count] := Path;
  Inc(Count);
end;

procedure TPlaylist.AddDir(const Path: WideString);
var
  i: Integer;
  tmp: array of WideString;
  dir: TDir;
begin
  SetLength(tmp, 32);
  i := 0;
  with TListDir.Create(Path) do
    try
      while Each(dir) do
        if not (dmDirectory in dir.Mode) then
        begin
          if i = Length(tmp) then
            SetLength(tmp, i + i div 4);
          tmp[i] := JoinPath([Path, dir.Name]);
          Inc(i);
        end;
    finally
      Free;
    end;
  SetLength(tmp, i);
  NLib.Sort(Items, CmpLogical);
  for i := 0 to High(tmp) do
    AddOne(tmp[i]);
end;

procedure TPlaylist.Clear;
begin
  Count := 0;
  SetLength(Items, 32);
end;

procedure TPlaylist.Sort;
begin
  SetLength(Items, Count);
  NLib.Sort(Items, CmpLogical);
end;

end.
